@@ -12,7 +12,7 @@ from peewee import ColumnBase
 from peewee import Expression
 from peewee import Node
 from peewee import NodeList
-from peewee import SENTINEL
+from peewee import __deprecated__
 from peewee import __exception_wrapper__
 
 try:
@@ -42,6 +42,7 @@ HCONTAINS_ANY_KEY = '?|'
 HKEY = '->'
 HUPDATE = '||'
 ACONTAINS = '@>'
+ACONTAINED_BY = '<@'
 ACONTAINS_ANY = '&&'
 TS_MATCH = '@@'
 JSONB_CONTAINS = '@>'
@@ -62,6 +63,9 @@ class _LookupNode(ColumnBase):
     def clone(self):
         return type(self)(self.node, list(self.parts))
 
+    def __hash__(self):
+        return hash((self.__class__.__name__, id(self)))
+
 
 class _JsonLookupBase(_LookupNode):
     def __init__(self, node, parts, as_json=False):
@@ -76,7 +80,9 @@ class _JsonLookupBase(_LookupNode):
         self._as_json = as_json
 
     def concat(self, rhs):
-        return Expression(self.as_json(True), OP.CONCAT, Json(rhs))
+        if not isinstance(rhs, Node):
+            rhs = Json(rhs)
+        return Expression(self.as_json(True), OP.CONCAT, rhs)
 
     def contains(self, other):
         clone = self.as_json(True)
@@ -131,14 +137,20 @@ class ObjectSlice(_LookupNode):
             parts = [value.start or 0, value.stop or 0]
         elif isinstance(value, int):
             parts = [value]
+        elif isinstance(value, Node):
+            parts = value
         else:
-            parts = map(int, value.split(':'))
+            # Assumes colon-separated integer indexes.
+            parts = [int(i) for i in value.split(':')]
         return cls(node, parts)
 
     def __sql__(self, ctx):
-        return (ctx
-                .sql(self.node)
-                .literal('[%s]' % ':'.join(str(p + 1) for p in self.parts)))
+        ctx.sql(self.node)
+        if isinstance(self.parts, Node):
+            ctx.literal('[').sql(self.parts).literal(']')
+        else:
+            ctx.literal('[%s]' % ':'.join(str(p + 1) for p in self.parts))
+        return ctx
 
     def __getitem__(self, value):
         return ObjectSlice.create(self, value)
@@ -218,6 +230,9 @@ class ArrayField(IndexedFieldMixin, Field):
     def contains_any(self, *items):
         return Expression(self, ACONTAINS_ANY, ArrayValue(self, items))
 
+    def contained_by(self, *items):
+        return Expression(self, ACONTAINED_BY, ArrayValue(self, items))
+
 
 class ArrayValue(Node):
     def __init__(self, field, value):
@@ -304,7 +319,9 @@ class JSONField(Field):
         return JsonPath(self, keys)
 
     def concat(self, value):
-        return super(JSONField, self).concat(Json(value))
+        if not isinstance(value, Node):
+            value = Json(value)
+        return super(JSONField, self).concat(value)
 
 
 def cast_jsonb(node):
@@ -441,8 +458,6 @@ class _empty_object(object):
         return False
     __bool__ = __nonzero__
 
-__named_cursor__ = _empty_object()
-
 
 class PostgresqlExtDatabase(PostgresqlDatabase):
     def __init__(self, *args, **kwargs):
@@ -456,25 +471,28 @@ class PostgresqlExtDatabase(PostgresqlDatabase):
             register_hstore(conn, globally=True)
         return conn
 
-    def cursor(self, commit=None):
+    def cursor(self, commit=None, named_cursor=None):
+        if commit is not None:
+            __deprecated__('"commit" has been deprecated and is a no-op.')
         if self.is_closed():
             if self.autoconnect:
                 self.connect()
             else:
                 raise InterfaceError('Error, database connection not opened.')
-        if commit is __named_cursor__:
-            return self._state.conn.cursor(name=str(uuid.uuid1()))
+        if named_cursor:
+            curs = self._state.conn.cursor(name=str(uuid.uuid1()))
+            return curs
         return self._state.conn.cursor()
 
-    def execute(self, query, commit=SENTINEL, named_cursor=False,
-                array_size=None, **context_options):
+    def execute(self, query, commit=None, named_cursor=False, array_size=None,
+                **context_options):
+        if commit is not None:
+            __deprecated__('"commit" has been deprecated and is a no-op.')
         ctx = self.get_sql_context(**context_options)
         sql, params = ctx.sql(query).query()
         named_cursor = named_cursor or (self._server_side_cursors and
                                         sql[:6].lower() == 'select')
-        if named_cursor:
-            commit = __named_cursor__
-        cursor = self.execute_sql(sql, params, commit=commit)
+        cursor = self.execute_sql(sql, params)
         if named_cursor:
             cursor = FetchManyCursor(cursor, array_size)
         return cursor
