@@ -5,6 +5,7 @@ except ImportError:
 from collections import namedtuple
 from inspect import isclass
 import re
+import warnings
 
 from peewee import *
 from peewee import _StringField
@@ -24,6 +25,10 @@ try:
     from playhouse import postgres_ext
 except ImportError:
     postgres_ext = None
+try:
+    from playhouse.cockroachdb import CockroachDatabase
+except ImportError:
+    CockroachDatabase = None
 
 RESERVED_WORDS = set([
     'and', 'as', 'assert', 'break', 'class', 'continue', 'def', 'del', 'elif',
@@ -220,7 +225,7 @@ class PostgresqlMetadata(Metadata):
         16: BooleanField,
         17: BlobField,
         20: BigIntegerField,
-        21: IntegerField,
+        21: SmallIntegerField,
         23: IntegerField,
         25: TextField,
         700: FloatField,
@@ -233,7 +238,7 @@ class PostgresqlMetadata(Metadata):
         1083: TimeField,
         1266: TimeField,
         1700: DecimalField,
-        2950: TextField, # UUID
+        2950: UUIDField, # UUID
     }
     array_types = {
         1000: BooleanField,
@@ -247,6 +252,7 @@ class PostgresqlMetadata(Metadata):
         1115: DateTimeField,
         1182: DateField,
         1183: TimeField,
+        2951: UUIDField,
     }
     extension_import = 'from playhouse.postgres_ext import *'
 
@@ -283,13 +289,13 @@ class PostgresqlMetadata(Metadata):
             postgres_ext.HStoreField)) if postgres_ext is not None else set()
 
         # Look up the actual column type for each column.
-        identifier = '"%s"."%s"' % (schema, table)
-        cursor = self.execute('SELECT * FROM %s LIMIT 1' % identifier)
+        identifier = '%s."%s"' % (schema, table)
+        cursor = self.execute(
+            'SELECT attname, atttypid FROM pg_catalog.pg_attribute '
+            'WHERE attrelid = %s::regclass AND attnum > %s', identifier, 0)
 
         # Store column metadata in dictionary keyed by column name.
-        for column_description in cursor.description:
-            name = column_description.name
-            oid = column_description.type_code
+        for name, oid in cursor.fetchall():
             column_types[name] = self.column_map.get(oid, UnknownField)
             if column_types[name] in extension_types:
                 self.requires_extension = True
@@ -315,6 +321,33 @@ class PostgresqlMetadata(Metadata):
         return super(PostgresqlMetadata, self).get_indexes(table, schema)
 
 
+class CockroachDBMetadata(PostgresqlMetadata):
+    # CRDB treats INT the same as BIGINT, so we just map bigint type OIDs to
+    # regular IntegerField.
+    column_map = PostgresqlMetadata.column_map.copy()
+    column_map[20] = IntegerField
+    array_types = PostgresqlMetadata.array_types.copy()
+    array_types[1016] = IntegerField
+    extension_import = 'from playhouse.cockroachdb import *'
+
+    def __init__(self, database):
+        Metadata.__init__(self, database)
+        self.requires_extension = True
+
+        if postgres_ext is not None:
+            # Attempt to add JSON types.
+            cursor = self.execute('select oid, typname, format_type(oid, NULL)'
+                                  ' from pg_type;')
+            results = cursor.fetchall()
+
+            for oid, typname, formatted_type in results:
+                if typname == 'jsonb':
+                    self.column_map[oid] = postgres_ext.BinaryJSONField
+
+            for oid in self.array_types:
+                self.column_map[oid] = postgres_ext.ArrayField
+
+
 class MySQLMetadata(Metadata):
     if FIELD_TYPE is None:
         column_map = {}
@@ -386,13 +419,13 @@ class SqliteMetadata(Metadata):
         'varchar': CharField,
     }
 
-    begin = '(?:["\[\(]+)?'
-    end = '(?:["\]\)]+)?'
+    begin = r'(?:["\[\(]+)?'
+    end = r'(?:["\]\)]+)?'
     re_foreign_key = (
-        '(?:FOREIGN KEY\s*)?'
-        '{begin}(.+?){end}\s+(?:.+\s+)?'
-        'references\s+{begin}(.+?){end}'
-        '\s*\(["|\[]?(.+?)["|\]]?\)').format(begin=begin, end=end)
+        r'(?:FOREIGN KEY\s*)?'
+        r'{begin}(.+?){end}\s+(?:.+\s+)?'
+        r'references\s+{begin}(.+?){end}'
+        r'\s*\(["|\[]?(.+?)["|\]]?\)').format(begin=begin, end=end)
     re_varchar = r'^\s*(?:var)?char\s*\(\s*(\d+)\s*\)\s*$'
 
     def _map_col(self, column_type):
@@ -402,7 +435,7 @@ class SqliteMetadata(Metadata):
         elif re.search(self.re_varchar, raw_column_type):
             field_class = CharField
         else:
-            column_type = re.sub('\(.+\)', '', raw_column_type)
+            column_type = re.sub(r'\(.+\)', '', raw_column_type)
             if column_type == '':
                 field_class = BareField
             else:
@@ -458,7 +491,13 @@ class Introspector(object):
 
     @classmethod
     def from_database(cls, database, schema=None):
-        if isinstance(database, PostgresqlDatabase):
+        if isinstance(database, Proxy):
+            if database.obj is None:
+                raise ValueError('Cannot introspect an uninitialized Proxy.')
+            database = database.obj  # Reference the proxied db obj.
+        if CockroachDatabase and isinstance(database, CockroachDatabase):
+            metadata = CockroachDBMetadata(database)
+        elif isinstance(database, PostgresqlDatabase):
             metadata = PostgresqlMetadata(database)
         elif isinstance(database, MySQLDatabase):
             metadata = MySQLMetadata(database)
@@ -485,7 +524,7 @@ class Introspector(object):
     def make_model_name(self, table, snake_case=True):
         if snake_case:
             table = make_snake_case(table)
-        model = re.sub('[^\w]+', '', table)
+        model = re.sub(r'[^\w]+', '', table)
         model_name = ''.join(sub.title() for sub in model.split('_'))
         if not model_name[0].isalpha():
             model_name = 'T' + model_name
@@ -502,7 +541,7 @@ class Introspector(object):
             column = re.sub('_id$', '', column) or column
 
         # Remove characters that are invalid for Python identifiers.
-        column = re.sub('[^\w]+', '_', column)
+        column = re.sub(r'[^\w]+', '_', column)
         if column in RESERVED_WORDS:
             column += '_'
         if len(column) and column[0].isdigit():
@@ -565,7 +604,7 @@ class Introspector(object):
 
             for col_name, column in table_columns.items():
                 if literal_column_names:
-                    new_name = re.sub('[^\w]+', '_', col_name)
+                    new_name = re.sub(r'[^\w]+', '_', col_name)
                 else:
                     new_name = self.make_column_name(col_name, col_name in fks,
                                                      snake_case)
@@ -646,12 +685,19 @@ class Introspector(object):
                 database = self.metadata.database
                 schema = self.schema
 
+        pending = set()
+
         def _create_model(table, models):
+            pending.add(table)
             for foreign_key in database.foreign_keys[table]:
                 dest = foreign_key.dest_table
 
                 if dest not in models and dest != table:
-                    _create_model(dest, models)
+                    if dest in pending:
+                        warnings.warn('Possible reference cycle found between '
+                                      '%s and %s' % (table, dest))
+                    else:
+                        _create_model(dest, models)
 
             primary_keys = []
             columns = database.columns[table]
@@ -669,7 +715,11 @@ class Introspector(object):
             # Fix models with multi-column primary keys.
             composite_key = False
             if len(primary_keys) == 0:
-                primary_keys = columns.keys()
+                if 'id' not in columns:
+                    Meta.primary_key = False
+                else:
+                    primary_keys = columns.keys()
+
             if len(primary_keys) > 1:
                 Meta.primary_key = CompositeKey(*[
                     field.name for col, field in columns.items()
@@ -698,7 +748,11 @@ class Introspector(object):
                         params['model'] = 'self'
                     else:
                         dest_table = column.foreign_key.dest_table
-                        params['model'] = models[dest_table]
+                        if dest_table in models:
+                            params['model'] = models[dest_table]
+                        else:
+                            FieldClass = DeferredForeignKey
+                            params['rel_model_name'] = dest_table
                     if column.to_field:
                         params['field'] = column.to_field
 
@@ -709,12 +763,14 @@ class Introspector(object):
                     constraint = SQL('DEFAULT %s' % column.default)
                     params['constraints'] = [constraint]
 
-                if column_name in column_indexes and not \
-                   column.is_primary_key():
-                    if column_indexes[column_name]:
-                        params['unique'] = True
-                    elif not column.is_foreign_key():
-                        params['index'] = True
+                if not column.is_primary_key():
+                    if column_name in column_indexes:
+                        if column_indexes[column_name]:
+                            params['unique'] = True
+                        elif not column.is_foreign_key():
+                            params['index'] = True
+                    else:
+                        params['index'] = False
 
                 attrs[column.name] = FieldClass(**params)
 
@@ -723,6 +779,9 @@ class Introspector(object):
             except ValueError:
                 if not skip_invalid:
                     raise
+            finally:
+                if table in pending:
+                    pending.remove(table)
 
         # Actually generate Model classes.
         for table, model in sorted(database.model_names.items()):
@@ -788,7 +847,7 @@ def get_table_sql(model):
         sql = sql.replace(model._meta.database.param, '%s')
 
     # Format and indent the table declaration, simplest possible approach.
-    match_obj = re.match('^(.+?\()(.+)(\).*)', sql)
+    match_obj = re.match(r'^(.+?\()(.+)(\).*)', sql)
     create, columns, extra = match_obj.groups()
     indented = ',\n'.join('  %s' % column for column in columns.split(', '))
 
